%
% Function: solab
%
% Purpose: Solves for the recursive representation of the stable solution to a system
% of linear difference equations.%
% Inputs: Two square matrices a and b and a natural number nk

%% a and b are the coefficient matrices of the difference equation
%% a*x(t+1) = b*x(t)
%% where x(t) is arranged so that the state variables come first, and
%
% nk is the number of state variables.
%
% Outputs: the decision rule f and the law of motion p. If we write
% x(t) = [k(t);u(t)] where k(t) contains precisely the state variables, then
% u(t)   = f*k(t) and
% k(t+1) = p*k(t).
%
% Calls: qzdiv

function [f,p] = solab(a,b,nk);

[aa,bb,q,z] = qz(a,b);            % upper triangular factorization of the matrix pencil b-za
[aa,bb,q,z] = qzdiv(1,aa,bb,q,z);   % reordering of generalized eigenvalues in ascending order

z21 = z(nk+1:end,1:nk);
z11 = z(1:nk,1:nk);

if rank(z11)<nk;
	error('Invertibility condition violated, solab.m line30')
end

z11i = z11\eye(nk);
aa11 = aa(1:nk,1:nk);
bb11 = bb(1:nk,1:nk);

if abs(bb(nk,nk))>abs(aa(nk,nk)) | abs(bb(nk+1,nk+1))<abs(aa(nk+1,nk+1));
   warning('Wrong number of stable eigenvalues.');
end

dyn = aa11\bb11;

f = real(z21*z11i);
p = real(z11*dyn*z11i);
end